angular.module('starter.controllers', [])

/*** TabsCtrl **********************************************************/
  .controller('TabsCtrl', ['$scope', 'TFCurrentMember', function($scope, TFCurrentMember) {
    var self = this;

    var currentMember = TFCurrentMember.getCurrentMember();
    self.admin = currentMember ? currentMember.admin : false;
    self.queuedReports = '';

    $scope.$on('MemberLoggedIn', function() {
      console.log('MemberLoggedIn');
      self.admin = TFCurrentMember.getCurrentMember().admin;
      //$scope.$apply();
    });

    $scope.$on('MemberLoggedOut', function() {
      console.log('MemberLoggedOut');
      self.admin = false;
      //$scope.$apply();
    });

    $scope.$on('ReportQueueUpdate', function(evt, count) {
      self.queuedReports = count > 0 ? count : '';
    });
  }])


/*** WriteReportCtrl **********************************************************/
  .controller('WriteReportCtrl', ['TFCurrentMember', 'TFCurrentReport', 'TFErrorLogging', function(TFCurrentMember, TFCurrentReport, TFErrorLogging) {
    setTimeout(TFErrorLogging.pushErrors,500);
    var self = this;

    self.currentMember = TFCurrentMember.getCurrentMember();
    self.report = TFCurrentReport.getCurrentReport();

    self.reportErrors = null;
    self.submitReport = function() {
      TFCurrentReport.saveCurrentReport()
        .then(null, null, function progress(done, total) {
          console.log(done, total)
        })
        .then(function success() {
          //console.log('success');
        })
        .catch(function error(err) {
          self.reportErrors = err.message;
        })
        ;
    };
  }])

/*** WorkReportCtrl *****************************************************/
  .controller('WorkCtrl', ['TFCurrentReport', function(TFCurrentReport) {
    var self = this;

    self.report = TFCurrentReport.getCurrentReport();

    console.dir(self.report);
  }])


/*** ViewReportCtrl **********************************************************/
  .controller('ViewReportCtrl', ['$q', '$scope', '$stateParams', 'TFReportsLocal', function($q, $scope, $stateParams, TFReportsLocal) {
    var self = this;

    self.uploadingString = '';

    TFReportsLocal.getReports({uuid: $stateParams.uuid}).then(function(reports) {
      if (reports.length === 1) {
        self.report = reports[0];
        if (self.report.upload_state == TFReportsLocal.STATE_UPLOADING) {
          if (self.report.data_upload_state == TFReportsLocal.STATE_UPLOADING) {
            self.uploadingString = 'Uploading report info...';
          }
          else {
            self.uploadingString = 'Uploading photo'+(self.report.photos.length == 1 ? '' : 's')+'...';
          }
        }
      }
      else {
        // uh oh, now what do we do?
      }
    });

    $scope.$on('TFReportsLocal.uploadreportdata.start', function(evt, state, report) {
      if (report.uuid === self.report.uuid) {
        self.uploadingString = 'Uploading report info...';
        self.report.upload_state = state;
      }
    });
    $scope.$on('TFReportsLocal.uploadreportdata.error', function(evt, state, report) {
      if (report.uuid === self.report.uuid) {
        self.uploadingString = '';
        self.report.upload_state = state;
      }
    });
    $scope.$on('TFReportsLocal.uploadreportdata.end', function(evt, state, report) {
      if (report.uuid === self.report.uuid) {
        self.uploadingString = '';
        self.report.upload_state = state;
      }
    });
    $scope.$on('TFReportsLocal.uploadreportphoto.start', function(evt, state, photo) {
      if (photo.reportuuid === self.report.uuid) {
        self.uploadingString = 'Uploading photo'+(self.report.photos.length == 1 ? '' : 's')+'...';
        self.report.upload_state = state;
      }
    });
    $scope.$on('TFReportsLocal.uploadreportphoto.error', function(evt, state, photo) {
      if (photo.reportuuid === self.report.uuid) {
        self.uploadingString = '';
        self.report.upload_state = state;
      }
    });
    $scope.$on('TFReportsLocal.uploadreportphoto.end', function(evt, state, photo) {
      if (photo.reportuuid === self.report.uuid) {
        self.uploadingString = '';
        self.report.upload_state = state;
      }
    });

  }])

/*** ReportsCtrl **************************************************************/
  .controller('ReportsCtrl', ['$q', '$scope', 'TFReportsLocal', function($q, $scope, TFReportsLocal) {
    var self = this;

    TFReportsLocal.refreshQueuedCount();

    $scope.$on('$ionicView.beforeEnter', function() {
      self.loadReports();
    });

    self.reports = [];
    self.reportsByTime = [];

    self.loadReports = function() {
      return TFReportsLocal.getReports()
        .then(function(reports) {
          self.reports = reports;
          self.reportsByTime = groupByTime(reports);
        })
        ;
    }

    function groupByTime(reports) {
      var arr = [];

      var lastHeading = '';
      var now = +new Date;

      for (var i = 0; i < reports.length; i++) {
        var newHeading = makeHeading(now, reports[i].ts);
        if (newHeading != lastHeading) {
          lastHeading = newHeading;
          arr.push({ time: lastHeading, reports: [reports[i]] });
        }
        else {
          arr[arr.length-1].reports.push(reports[i]);
        }
      }

      return arr;
    }

    function makeHeading(now, ts) {
      var d = new Date();
      d.setTime(ts);

      var diff = now - ts;

      if (diff < 5 * 60 * 1000) {
        return 'Just Now';
      }

      if (diff < 60 * 60 * 1000) {
        return 'Last Hour';
      }

      var today = new Date();
      today.setHours(0);
      today.setMinutes(0);
      today.setSeconds(0);

      if (d.toLocaleDateString() === today.toLocaleDateString()) {
        return 'Today';
      }

      var yesterday = new Date();
      yesterday.setTime(yesterday.getTime() - 24 * 60 * 60 * 1000);
      yesterday.setHours(0);
      yesterday.setMinutes(0);
      yesterday.setSeconds(0);

      if (d.toLocaleDateString() === yesterday.toLocaleDateString()) {
        return 'Yesterday';
      }

      return d.toLocaleDateString();
    }

    self.refresh = function() {
      self.loadReports()
        .finally(function() {
          $scope.$broadcast('scroll.refreshComplete');
          TFReportsLocal.refreshQueuedCount();
        })
        .finally(TFReportsLocal.uploadReports)
        ;
    }

    //$scope.$on('TFReportsLocal.uploadreports.start', function(evt, state, report) {
    //  console.log('TFReportsLocal.uploadreports.start');
    //});
    //$scope.$on('TFReportsLocal.uploadreports.end', function(evt, state, report) {
    //  console.log('TFReportsLocal.uploadreports.end');
    //});
    //$scope.$on('TFReportsLocal.uploadreport.start', function(evt, state, report) {
    //  console.log('TFReportsLocal.uploadreport.start');
    //});
    //$scope.$on('TFReportsLocal.uploadreport.end', function(evt, state, report) {
    //  console.log('TFReportsLocal.uploadreport.end');
    //});

    $scope.$on('TFReportsLocal.uploadreport.end', function(evt, state, report) {
      // could be smarter, but just reload from db for now
      self.loadReports();
    });

    $scope.$on('TFReportsLocal.uploadreportdata.start', function(evt, state, report) {
      //console.log('TFReportsLocal.uploadreportdata.start');
      reportUploadStatusChange(report, state);
    });
    $scope.$on('TFReportsLocal.uploadreportdata.error', function(evt, state, report) {
      //console.log('TFReportsLocal.uploadreportdata.error');
      reportUploadStatusChange(report, state);
    });
    $scope.$on('TFReportsLocal.uploadreportdata.end', function(evt, state, report) {
      //console.log('TFReportsLocal.uploadreportdata.end');
      reportUploadStatusChange(report, state);
    });
    $scope.$on('TFReportsLocal.uploadreportphoto.start', function(evt, state, photo) {
      //console.log('TFReportsLocal.uploadreportphoto.start');
      photoUploadStatusChange(photo, state);
    });
    $scope.$on('TFReportsLocal.uploadreportphoto.error', function(evt, state, photo) {
      //console.log('TFReportsLocal.uploadreportphoto.error');
      photoUploadStatusChange(photo, state);
    });
    $scope.$on('TFReportsLocal.uploadreportphoto.end', function(evt, state, photo) {
      //console.log('TFReportsLocal.uploadreportphoto.end');
      photoUploadStatusChange(photo, state);
    });

    function findReport(reportuuid) {
      for (var i = 0; i < self.reports.length; i++) {
        if (self.reports[i].uuid == reportuuid) {
          var report = self.reports[i];
          return report;
        }
      }
    }

    function reportUploadStatusChange(report, state) {
      var report = findReport(report.uuid);
      if (report) {
        report.upload_state = state;
      }
    }

    function photoUploadStatusChange(photo, state) {
      var report = findReport(photo.reportuuid);
      if (report) {
        report.upload_state = state;
      }
    }
  }])

/*** AccountCtrl *************************************************************/

  .controller('AccountCtrl', ['TFAPI', 'TFCurrentMember', function(TFAPI, TFCurrentMember) {
    var self = this;

    self.errors = false;
    self.loggingIn = false;
    self.currentMember = TFCurrentMember.getCurrentMember();
    self.endpointKey = TFAPI.CURRENT_ENDPOINT_KEY

    self.loginDetails = {
      username: '',
      password: '',
    };

    self.submitLoginBentomas = function() {
      self.loginDetails.username = 'bentomas';
      self.loginDetails.password = 'va';

      self.submitLogin();
    }

    self.submitLogin = function() {
      self.loggingIn = true;
      TFCurrentMember.login(self.loginDetails).then(function(member) {
        //success
        self.errors = false;
        self.loggingIn = false;
        self.currentMember = member;
        self.loginDetails.password = '';
      }, function(err) {
        // error
        self.errors = err.message;
        self.loggingIn = false;
      });
    }

    self.submitLogout = function() {
      TFCurrentMember.logout()
      self.currentMember = TFCurrentMember.getCurrentMember();
    }
  }])

/*** ChooseTrailCtrl **********************************************************/
  .controller('ChooseTrailCtrl', ['$scope', 'TFCurrentReport', 'TFTrailsSearch', function($scope, TFCurrentReport, TFTrailsSearch) {
    var self = this;

    var distanceHeadings = [1, 5, 10, 25, 50, 100, 500, 1000, 1500];

    self.searching = true;
    self.refreshing = false;
    self.error = false;
    self.offline = false;

    self.search = {
      text:  '',
      range: 5000,
      location: TFTrailsSearch.CURRENT_LOCATION,
    };
    self.trails = [];

    self.refresh = function() {
      self.refreshing = true;

      self.runSearch(true).finally(function() {
        self.refreshing = false;
        $scope.$broadcast('scroll.refreshComplete');
      });
    }

    self.runSearch = function(skipCache) {
      self.searching = true;
      return TFTrailsSearch.search(self.search, skipCache)
        .then(function(result) {
          self.searching = false;
          self.offline = result.source == 'cache' && result.reason ? result.reason : false;
          console.log('source', result.source);
          console.log('offline', self.offline);
          self.trails = result.data;
          self.error = false;
        })
        .catch(function(err) {
          self.searching = false;
          self.offline = err.message == 'Network Unavailable' || err.message == 'Could not reach server' ? err.message : false;
          self.error = 'search failed';
        })
        ;
    }

    self.selectTrail = function(trail) {
      TFCurrentReport.getCurrentReport().trail = trail;
    }

    $scope.$on('$ionicView.afterEnter', self.runSearch);
  }])

/*** ChooseStatusCtrl *********************************************************/
  .controller('ChooseStatusCtrl', ['TFCurrentReport', 'TFStatuses', function(TFCurrentReport, TFStatuses) {
    var self = this;

    self.statuses = function() {
      return TFStatuses.all();
    };

    self.selectStatus = function(status) {
      TFCurrentReport.getCurrentReport().status = status;
    };
  }])

/*** ChooseConditionCtrl ******************************************************/
  .controller('ChooseConditionCtrl', ['TFCurrentReport', 'TFConditions', function(TFCurrentReport, TFConditions) {
    var self = this;

    self.conditions = function() {
      return TFConditions.all();
    };

    self.selectCondition = function(condition) {
      TFCurrentReport.getCurrentReport().condition = condition;
    };
  }])

/*** WriteDescriptionCtrl *****************************************************/
  .controller('WriteDescriptionCtrl', ['$scope', '$timeout', 'TFCurrentReport', 'TFReportQuickSuggestions', function($scope, $timeout, TFCurrentReport, TFReportQuickSuggestions) {
    var self = this;

    self.suggestions = TFReportQuickSuggestions.all();
    self.quickSuggestion = '';

    $scope.$watch(function() {
      return self.quickSuggestion;
    }, function() {
      self.currentReport.description = (self.currentReport.description + ' ' + self.quickSuggestion).trim();
      if (self.currentReport.description) {
        self.currentReport.description += ' ';
      }
      setTimeout(function() {
        self.quickSuggestion = '';
        $scope.$apply();
      }, 10)
    });

    self.currentReport = TFCurrentReport.getCurrentReport();
  }])

/*** SettingsCtrl *****************************************************/
  .controller('SettingsCtrl', ['TFAPI', 'PBRMS', function(TFAPI, PBRMS) {
    var self = this;

    // need to move this to app.js
    self.version = appVersion;

    self.apiEndpoints = TFAPI.API_ENDPOINTS;
    self.currentEndpoint = TFAPI.CURRENT_ENDPOINT_KEY;

    self.selectEndpoint = function(key) {
      TFAPI.setEndpoint(key);
      PBRMS.setEndpoint(key);
      self.currentEndpoint = TFAPI.CURRENT_ENDPOINT_KEY;
    }

  }])
  
/*** ViewNearbyReportsCtrl **********************************************************/
  .controller('ViewNearbyReportsCtrl', ['$scope', 'TFReportsNearby', function($scope, TFReportsNearby) {
    var self = this;

    self.error = false;
    self.offline = false;
    self.reports = [];
    self.searching = true;

    self.search = {
      range: 50,
      location: TFReportsNearby.CURRENT_LOCATION,
    };

    self.refresh = function() {
      self.refreshing = true;

      self.runSearch(true).finally(function() {
        self.refreshing = false;
        $scope.$broadcast('scroll.refreshComplete');
      });
    };

    self.runSearch = function() {
      return TFReportsNearby.search(self.search)
        .then(function(result) {
          self.searching = false;
          self.error = false;
          self.offline = result.source == 'cache' && result.reason ? result.reason : false;
          self.reports = result.data;
        })
        .catch(function(err) {
          self.searching = false;
          console.log('error running nearby reports search', err.message);
          self.offline = err.message == 'Network Unavailable' || err.message == 'Could not reach server' ? err.message : false;
          self.error = 'search failed: '+err.message;
        })
        ;
    };

    $scope.$on('$ionicView.afterEnter', self.runSearch);
  }])

