angular.module('starter.services')
  .service('PhoneSQLiteCache', ['$q', 'PhoneSQLite', function($q, PhoneSQLite) {
    var self = this;

    var DB_NAME = 'cache.db';
    var TABLE_NAME = 'cache';

    self.get = function(key) {
      return PhoneSQLite.query(DB_NAME,'SELECT * FROM '+TABLE_NAME+' WHERE key = ?', [key]).then(function(results) {
        if (results.rows.length > 0) {
          var row = results.rows.item(0);

          try {
            row.value = JSON.parse(row.value);
          }
          catch(err) {
            return $q.reject(new Error ('Cannot parse cached value'));
          }

          return row;
        }
        else {
          return $q.reject('Not in cache');
        }
      }, function(err) {
        return self.createCacheTable()
          .then(function() {
            return self.get(key);
          });
      });
    };

    self.createCacheTable = function() {
      var query = 'CREATE TABLE '+TABLE_NAME+'(key PRIMARY KEY, time, value);';
      return PhoneSQLite.query(DB_NAME, query)
        .catch(function(err) {
          return $q.reject(new Error('Cannot access cache'));
        })
    }


    self.put = function(key, value) {
      var json_value = JSON.stringify(value);
      var query = 'INSERT OR REPLACE INTO '+TABLE_NAME+' (key, time, value) VALUES (?,?,?)';
      var params = [key, +new Date, json_value];

      // TODO: maybe don't wait for the insert to finish? and just return the value?
      return PhoneSQLite.query(DB_NAME, query, params)
        .then(function() {
          return value;
        })
    }
  }]);
