angular.module('starter.services')
  .service('PhoneSQLite', ['$q', function($q) {
    // Wait for Cordova to load
    var self = this;
    self.dbs = {};

    var deviceReady = false;
    var deviceReadyDeferred = $q.defer();
    var deviceReadyPromise  = deviceReadyDeferred.promise;
    document.addEventListener("deviceready", onDeviceReady, false);
    function onDeviceReady() {
      deviceReady = true;
      deviceReadyDeferred.resolve();
    }

    self.query = function(dbName, query, params) {
      if (!deviceReady) {
        return deviceReadyPromise.then(function() {
          return self.query(dbName, query, params);
        });
      }
      else {
        var deferred = $q.defer();

        if (!params || params.constructor !== Array) {
          params = [];
        }

        if (!(dbName in self.dbs)) {
          self.dbs[dbName] = window.sqlitePlugin.openDatabase({name: dbName});
        }
        var db = self.dbs[dbName];

        //console.log('query');
        //console.log(query);
        //console.dir(params);

        db.transaction(function (tx) {
          tx.executeSql(query, params, function(ts, results) {
            deferred.resolve(results);
          }, function error(tx, err) {
            deferred.reject(err);
          });
        });

        return deferred.promise;
      }
    }

    self.fetchrows = function(dbName, query, params) {
      return self.query(dbName, query, params).then(function(results) {
        var arr = [];
        for (var i = 0; i < results.rows.length; i++) {
          arr.push(results.rows.item(i));
        }
        return arr;
      });
    }

    self.fetchrow = function(dbName, query, params) {
      return self.query(dbName, query, params).then(function(results) {
        if (results.rows.length) {
          return results.rows.item(0);
        }
        else {
          return null;
        }
      });
    }

    self.createTableIfDoesNotExistWrapper = function(fn, db, tableName, schema) {
      return function() {
        var obj = this;
        var args = arguments;

        return $q.when(fn.apply(obj, args))
          .catch(function(err) {
            // make sure there is at least a space at the end
            err.message += ' ';
            if (err.message.indexOf('no such table: '+tableName +' ') === 0) {
              return self.query(db, schema)
                .then(function() {
                  return fn.apply(obj, args);
                }).catch(function(err) {
                  console.log('could not create table "'+tableName+'"')
                  console.log(err.message)
                });
            }
            else {
              return $q.reject(err);
            }
          });
      }
    }

  }]);
