angular.module('starter.services')
  .service('TFAPI', ['$http', '$q', 'PhoneLocalStorage', 'PhoneSQLiteCache', 'TFErrorLogging', function($http, $q, PhoneLocalStorage, PhoneSQLiteCache, TFErrorLogging) {
    var self = this;

    var COULD_NOT_REACH_SERVER_ERROR = self.COULD_NOT_REACH_SERVER_ERROR = 'Could Not Reach Server';
    var SERVER_ERROR                 = self.SERVER_ERROR                 = 'Server Error';
    var API_ERROR                    = self.API_ERROR                    = 'API Error';

    self.API_ENDPOINTS    = {
      'production': {
        domain:     'http://www.trailforks.com',
        app_id:     3,
        app_secret: '48ebb5b701a1cb2c',
      },
      'radek': {
        domain:     'http://www.va2.trailforks.com',
        app_id:     3,
        app_secret: '48ebb5b701a1cb2c',
      },
      'trevor': {
        domain:     'http://www.va5.trailforks.com',
        app_id:     3,
        app_secret: '48ebb5b701a1cb2c',
      },
      'ben': {
        domain:     'http://www.va6.trailforks.com',
        app_id:     3,
        app_secret: '48ebb5b701a1cb2c',
      },
    };

    self.CURRENT_ENDPOINT_KEY = PhoneLocalStorage.get('endpoint', 'production');

    var REQUEST_TIMEOUT = 10000;
    var REQUEST_TIMEOUT = 5000;
    //var REQUEST_TIMEOUT = 1000;

    var CACHE_TIMEOUT = 900; // 15 minutes
    var CACHE_TIMEOUT = 300; // 5 minutes
    var CACHE_TIMEOUT = 180; // 3 minutes
    //var CACHE_TIMEOUT = 10; // 5 seconds

    // convert to milliseconds
    CACHE_TIMEOUT = CACHE_TIMEOUT * 1000;

    self.publicAPI = function(action, data, options) {
      var promise, err, source, reason;
      if (options === undefined) {
        options = {};
      }

      if (options.cache) {
        promise = PhoneSQLiteCache.get(options.cache)
          .then(function(cached) {
            // it's in the cache!
            if (!options.readFromCache || cached.time < ((+new Date)-CACHE_TIMEOUT)) {
              // but expired or they wanted to load a new version if we had one
              return request().catch(function(err) {
                // oops, there was an error fetching it, so return the cached result
                // anyway
                source = 'cache';
                reason = err.message;
                return cached.value;
              });
            }
            else {
              reason = '';
              source = 'cache';
              return cached.value;
            }
          })
          .catch(function() {
            // couldn't get it from the cache, so just request it!
            return request();
          })
          .then(function(data) {
            if (source !== 'cache') {
              return PhoneSQLiteCache.put(options.cache, data);
            }
            else {
              return data;
            }
          })
      }
      else {
        promise = request();
      }

      // finally, make sure we mark where we got the data from, and why
      return promise
        .then(function(data) {
          return {
            data: data,
            source: source,
            reason: reason,
          }
        });

      function request() {
        var network_state = navigator.connection.type;
        var offline = ((network_state == 'unknown') || (network_state == 'none')) ? true : false;

        if (offline) {
          return $q.reject(new Error('Network Unavailable'));
        }
        else {
          var url = self.API_ENDPOINTS[self.CURRENT_ENDPOINT_KEY].domain+'/api/1/'+action+'?app_id='+self.API_ENDPOINTS[self.CURRENT_ENDPOINT_KEY].app_id+'&app_secret='+self.API_ENDPOINTS[self.CURRENT_ENDPOINT_KEY].app_secret;
          var request = { method: 'GET', url: url, params: data, timeout: REQUEST_TIMEOUT };
          return $http(request)
            .then(function(response) {
              if (response.data.error) {
                return $q.reject(response);
              }
              else {
                source = 'network';
                return response.data.data;
              }
            })
            // more helpful error response
            .catch(function(err_response) {
              var msg;

              var status = err_response.status;
              if (status === 0) {
                msg = COULD_NOT_REACH_SERVER_ERROR;
              }
              else if (status >= 500 && status < 600) {
                msg = SERVER_ERROR;
              }
              // for any status reponse between 200 and 500 (so 2xx and 4xx responses),
              // look for an error message in the returned data
              else if (status >= 200 && status < 500) {
                if ('data' in err_response && 'message' in err_response.data) {
                  msg = err_response.data.message;
                }
                else {
                  msg = API_ERROR;
                }
              }

              //console.log('Public API Error: '+msg);
              //console.log('request');
              //console.dir(request);
              //console.log('response');
              //console.dir(err_response);

              var err = new Error(msg);
              err.status = status;
              err.code   = err_response.code;

              return TFErrorLogging.log('TFAPI.publicAPI', err, {request: request, code: err.code})
            });
        }
      }
    };

    self.privateAPI = function(action, data, tokens) {
      data['app_id'] = self.API_ENDPOINTS[self.CURRENT_ENDPOINT_KEY].app_id;
      //data['app_secret'] = self.API_ENDPOINTS[self.CURRENT_ENDPOINT_KEY].app_secret;

      var timestamp = Math.round(+new Date()/1000);
      data['timestamp'] = timestamp;
      data['token_public'] = tokens.public;
      data['hash'] = SHA1(timestamp+tokens.private);

      var url = self.API_ENDPOINTS[self.CURRENT_ENDPOINT_KEY].domain+'/api/1/'+action;
      var request = {
        method:           'POST',
        url:              url,
        data:             data,
        headers:          {'content-type': 'application/x-www-form-urlencoded'},
        transformRequest: convertToPostData,
      }

      return $http(request).then(function(response) {
        if (response.data.error !== 0) {
          var message = response.data.message;
          if (typeof message !== 'string') {
            if (message && message.message) {
              message = message.message;
            }
            else {
              message = '';
            }
          }
          return $q.reject(new Error(message));
        }
        else {
          return response.data.data;
        }
      })
      .catch(function(err_response) {
        var msg, contentid;

        var status = err_response.status;
        if (status === 0) {
          msg = COULD_NOT_REACH_SERVER_ERROR;
        }
        else if (status >= 500 && status < 600) {
          msg = SERVER_ERROR;
        }
        else if (status >= 400 && status < 500) {
          if ('data' in err_response && 'message' in err_response.data) {
            msg = err_response.data.message;
          }
          else {
            msg = API_ERROR;
          }
          if (status == 406) {
            contentid = err_response.data.data.contentid;
          }
        }

        //console.log('Private API Error: '+msg);
        //console.log('request');
        //console.dir(request);
        //console.log('response');
        //console.dir(err_response);

        var err = new Error(msg);
        err.status    = status;
        err.code      = err_response.code;
        err.contentid = contentid;

        return TFErrorLogging.log('TFAPI.privateAPI', err, {request: request, code: err.code, contentid: err.contentid})
      });
    }

    function convertToPostData(data, headers) {
      var requestStr;
      for (var key in data) {
        if (requestStr) {
          requestStr += '&' + key + '=' + data[key];
        } else {
          requestStr = key + '=' + data[key];
        }
      }
      return requestStr;
    }

    self.privateUpload = function(file_path, data, tokens, progressCallback) {
      var deferred = $q.defer();

      data['app_id'] = self.API_ENDPOINTS[self.CURRENT_ENDPOINT_KEY].app_id;
      //data['app_secret'] = self.API_ENDPOINTS[self.CURRENT_ENDPOINT_KEY].app_secret;

      var timestamp = Math.round(+new Date()/1000);
      data['timestamp'] = timestamp;
      data['token_public'] = tokens.public;
      data['hash'] = SHA1(timestamp+tokens.private);

      var url = self.API_ENDPOINTS[self.CURRENT_ENDPOINT_KEY].domain+'/api/1/photo';

      var request = new FileUploadOptions();
      request.fileKey = 'Filedata';
      request.fileName = file_path.substr(file_path.lastIndexOf('/') + 1);

      request.params = {};
      for (var key in data) {
        request.params[key] = data[key];
      }

      var ft = new FileTransfer();
      ft.onprogress = progress;
      ft.upload(file_path, encodeURI(url), done, failed, request);

      function done(response) {
        if (response.responseCode == 200) {
          deferred.resolve(response.response);
        }
        else {
          deferred.reject();
        }
      }

      function failed(err_response)  {
        var msg, contentid;

        var status = err_response.http_status;
        if (status === 0) {
          msg = COULD_NOT_REACH_SERVER_ERROR;
        }
        else if (status >= 500 && status < 600) {
          msg = SERVER_ERROR;
        }
        else if (status >= 400 && status < 500) {
          var body = err_response.body;
          var json;

          try {
            json = JSON.parse(body);
            msg = json.message;
            if (status === 406) {
              contentid = json.data.contentid;
            }
          }
          catch(err) {
            msg = API_ERROR;
          }
        }

        var err = new Error(msg);
        err.status    = status;
        err.code      = err_response.code;
        err.contentid = contentid;

        //console.log('Private Upload Error: '+msg);
        //console.log('request');
        //console.dir(request);
        //console.log('response');
        //console.dir(err_response);

        return TFErrorLogging.log('TFAPI.privateUpload', err, {request: request, response: err_response, code: err.code, contentid: err.contentid})
          .finally(function() {
            deferred.reject(err);
          })
      }

      function progress(e) {
        if (e.lengthComputable) {
          deferred.notify(e.loaded / e.total);
        }
      }

      return deferred.promise;
    }

    self.setEndpoint = function(key) {
      self.CURRENT_ENDPOINT_KEY = key;
      PhoneLocalStorage.set('TFAPI-endpoint', self.CURRENT_ENDPOINT_KEY);
    }
  }]);
