angular.module('starter.services')
    .service('TFCurrentMember', ['$q', '$rootScope', 'PhoneLocalStorage', 'TFAPI', function($q, $rootScope, PhoneLocalStorage, TFAPI) {
      var self = this;

      var currentMember = null;
      var tokens = null;

      var adminIDs = [
        1,      // radek
        493963, // ben
        454369, // trevor
      ];

      function setLoginState(userid, username, token_private, token_public) {
        if (userid && username && token_private && token_public) {
          currentMember = {
            username: username,
            userid:   userid,
          }

          tokens = {
            public:  token_public,
            private: token_private,
          }

          currentMember.admin = adminIDs.indexOf(currentMember.userid) >= 0;

          $rootScope.$broadcast('MemberLoggedIn');
        }
      }

      self.checkForLoggedInUser = function() {
        var userid        = PhoneLocalStorage.get('userid');
        var username      = PhoneLocalStorage.get('username');
        var token_private = PhoneLocalStorage.get('token_private');
        var token_public  = PhoneLocalStorage.get('token_public');

        if (userid && username && token_private && token_public) {
          setLoginState(userid, username, token_private, token_public);
        }
      }

      self.login = function(loginDetails) {
        return TFAPI.publicAPI('login', loginDetails)
          .then(function (response) {
            var userid = Number(response.data.userid);
            var username = loginDetails.username;
            var token_private = response.data.token_private;
            var token_public = response.data.token_public;

            setLoginState(userid, username, token_private, token_public);

            if (token_private) {
              PhoneLocalStorage.set("token_private", token_private);
            }
            if (token_public) {
              PhoneLocalStorage.set("token_public", token_public);
            }
            if (userid) {
              PhoneLocalStorage.set("userid", userid);
            }
            if (username) {
              PhoneLocalStorage.set("username", username);
            }

            return currentMember;
          });
      };

      self.logout = function() {
        currentMember = null;
        tokens = null;

        PhoneLocalStorage.remove('username');
        PhoneLocalStorage.remove('userid');
        PhoneLocalStorage.remove('token_private');
        PhoneLocalStorage.remove('token_public');

        $rootScope.$broadcast('MemberLoggedOut');
      }

      self.getCurrentMember = function() {
        return currentMember;
      }

      self.postPrivateAPI = function(action, data) {
        if (!tokens) {
          return $q.reject();
        }
        else {
          data.userid = currentMember.userid;
          return TFAPI.privateAPI(action, data, tokens);
        }
      }

      self.uploadPrivateAPI = function(file_path, data) {
        if (!tokens) {
          return $q.reject();
        }
        else {
          data.userid = currentMember.userid;
          return TFAPI.privateUpload(file_path, data, tokens);
        }
      }

      self.checkForLoggedInUser();
    }]);

