angular.module('starter.services')
  .service('TFErrorLogging', ['$http', '$q', 'PhoneSQLite', 'PBRMS', function($http, $q, PhoneSQLite, PBRMS) {
    var self = this;

    var MAX_UPLOADATTEMPTS = 10;
    var DB_NAME = 'tf.db';
    var ERRORS_TABLE_SCHEMA = 'CREATE TABLE errors(uuid PRIMARY KEY, source TEXT, message TEXT, stack TEXT, window_location TEXT, ts, url TEXT, lineNo, col, extra TEXT, uploaded DEFAULT 0, upload_attempts DEFAULT 0);';

    self.log = function(source, err, extra) {
      if (err.uuid !== undefined) {
        // this error has already been logged, so just reject it and move along
        return $q.reject(err);
      }
      else {
        //console.log('TFErrorLogging.log');

        if (err instanceof Error) {
          var details = {
            message: err.message,
            stack:   err.stack,
            lineNo:  err.line || err.lineNumber,
            url:     err.fileName,
            col:     err.column || err.columnNumber,
          };
        }
        else {
          details = {};
        }

        details.uuid = uuidgen.v1();
        details.source = source;

        details.windowLocation = window.location+'';

        details.extra = extra;

        // try pushing errors in a little bit
        setTimeout(self.pushErrors,50);

        return self.logErrorDetails(details)
          .then(function(result) {
            err.uuid = details.uuid;
          })
          .catch(function(e) {
            console.log('error inserting error into db');
            console.dir(e);
          })
          .finally(function() {
            return $q.reject(err);
          })
      }
    }

    self.logErrorDetails = function(details) {
        var query = 'INSERT INTO errors (uuid, message, stack, lineNo, url, col, window_location, ts, source, extra) VALUES(?,?,?,?,?,?,?,?,?,?)';
        var params = [details.uuid, details.message, details.stack, details.lineNo, details.url, details.col, details.windowLocation, +new Date, details.source, JSON.stringify(details.extra)];
        return PhoneSQLite.query(DB_NAME, query, params)
    }

    self.pushErrors = function() {
      var errors;
      var index = 0;

      var query = 'SELECT * FROM errors WHERE uploaded = 0 AND upload_attempts < ? ORDER BY ts DESC';
      var params = [MAX_UPLOADATTEMPTS];
      PhoneSQLite.fetchrows(DB_NAME, query, params)
        .then(function(rows) {
          errors = rows;

          pushNextError();
        });

      function pushNextError() {
        if (index < errors.length) {
          var error = errors[index];
          index++;

          var data = {
            details: JSON.stringify(error),
          }

          var url = PBRMS.RMS_ENDPOINTS[PBRMS.CURRENT_ENDPOINT_KEY].domain+'/system/jserror';
          var request = {
            method:           'POST',
            url:              url,
            params:           { 'skip-mobile-redirect': 1 },
            data:             data,
            headers:          {'content-type': 'application/x-www-form-urlencoded'},
            transformRequest: convertToPostData,
          }

          return $http(request)
            .then(function() {
              var query = 'UPDATE errors SET uploaded = 1 WHERE uuid = ?';
              var params = [error.uuid];
              return PhoneSQLite.query(DB_NAME, query, params)
            })
            .catch(function(err_response) {
              console.log('error pushing error');
              console.dir(err_response);

              // do nothing when this fails. We'll try again later

              var query = 'UPDATE errors SET upload_attempts = upload_attempts+1 WHERE uuid = ?';
              var params = [error.uuid];
              return PhoneSQLite.query(DB_NAME, query, params)
            })
            //.then(pushNextError)
        }
      }
    }

    function convertToPostData(data, headers) {
      var requestStr;
      for (var key in data) {
        if (requestStr) {
          requestStr += '&' + key + '=' + encodeURIComponent(data[key]);
        } else {
          requestStr = key + '=' + encodeURIComponent(data[key]);
        }
      }
      return requestStr;
    }

    self.logErrorDetails = PhoneSQLite.createTableIfDoesNotExistWrapper(self.logErrorDetails,  DB_NAME, 'errors', ERRORS_TABLE_SCHEMA);
  }]);
