angular.module('starter.services')
  .service('TFReportsLocal', ['$q', '$rootScope', 'PhoneFileSystem', 'PhoneSQLite', 'TFTrailsLocal', 'TFCurrentMember', 'TFStatuses', 'TFConditions', 'TFUploadLogging', 'TFAPI', function($q, $rootScope, PhoneFileSystem, PhoneSQLite, TFTrailsLocal, TFCurrentMember, TFStatuses, TFConditions, TFUploadLogging, TFAPI) {
    var self = this;

    var STATE_QUEUED    = self.STATE_QUEUED    = 0;
    var STATE_UPLOADED  = self.STATE_UPLOADED  = 1;
    var STATE_UPLOADING = self.STATE_UPLOADING = 2;
    var STATE_ERROR     = self.STATE_ERROR     = 3;

    // how long to wait for an upload to finish before assuming it failed
    // and trying again
    var UPLOADING_TIMEOUT = 5; // in minutes
    UPLOADING_TIMEOUT = UPLOADING_TIMEOUT * 60 * 1000; // convert to milliseconds

    self.numQueued = -1;

    var DB_NAME = 'tf.db';

    var REPORTS_TABLE_SCHEMA = 'CREATE TABLE reports(id TEXT PRIMARY KEY, uuid TEXT, trailid, ts, status, condition, description, marker, userid, trail_work DEFAULT 0, work_hours DEFAULT 0, work_attendance DEFAULT 0, upload_state DEFAULT 0, upload_ts, numPhotos DEFAULT 0, numPhotosUploaded DEFAULT 0);';
    var REPORT_PHOTOS_TABLE_SCHEMA = 'CREATE TABLE report_photos(uuid TEXT PRIMARY KEY, reportid TEXT, reportuuid TEXT, name, upload_state DEFAULT 0, upload_ts);';

    function getStatusesForReports(reports) {
      for (var i = 0; i < reports.length; i++) {
        reports[i].status = TFStatuses.get(reports[i].status);
      }
      return reports;
    }

    function getConditionsForReports(reports) {
      for (var i = 0; i < reports.length; i++) {
        reports[i].condition = TFConditions.get(reports[i].condition);
      }
      return reports;
    }

    function getTrailsForReports(reports) {
      var trailids = [];
      for (var i = 0; i < reports.length; i++) {
        trailids.push(reports[i].trailid);
      }
      return TFTrailsLocal.getTrails(trailids).then(function(trails) {
        for (var i = 0; i < reports.length; i++) {
          for (var j = 0; j < trails.length; j++) {
            if (reports[i].trailid === trails[j].trailid) {
              reports[i].trail = trails[j];
              break;
            }
          }
        }
        return reports;
      });
    }

    function getPhotosForReports(reports) {
      if (reports.length < 1) {
        return reports;
      }
      for (var i = 0; i < reports.length; i++) {
        reports[i].photos = [];
      }

      var params = [];
      var placeholders = [];
      for (var i = 0; i < reports.length; i++) {
        params.push(reports[i].uuid);
        placeholders.push('?');
      }

      var query = 'SELECT * FROM report_photos WHERE CAST(reportuuid AS text ) IN ('+placeholders.join(',')+');';
      return PhoneSQLite.fetchrows(DB_NAME, query, params).then(function(rows) {
        for (var i = 0; i < reports.length; i++) {
          for (var j = 0; j < rows.length; j++) {
            var row = rows[j];
            if (row.reportuuid === reports[i].uuid) {
              row.path = cordova.file.dataDirectory + row.name;
              reports[i].photos.push(row);
            }
          }
        }
        return reports;
      });
    }

    function matchUploadStateToPhotos(reports) {
      for (var i = 0; i < reports.length; i++) {
        var state = reports[i].upload_state;
        reports[i].data_upload_state = state;
        for (var j = 0; j < reports[i].photos.length; j++) {
          var p = reports[i].photos[j];
          if (state === STATE_UPLOADING || p.upload_state === STATE_UPLOADING) {
            state = STATE_UPLOADING;
          }
          else if (state === STATE_ERROR || p.upload_state === STATE_ERROR) {
            state = STATE_ERROR;
          }
          else if (state === STATE_QUEUED || p.upload_state === STATE_QUEUED) {
            state = STATE_QUEUED;
          }
          else {
            state = STATE_UPLOADED;
          }
        }

        reports[i].upload_state = state;
      }
      return reports;
    }

    function convertTrailWork(reports) {
      for (var i = 0; i < reports.length; i++) {
        reports[i].work = {
          worked: (reports[i].trail_work == 1),
          numPeople: reports[i].work_attendance,
          numHours:  reports[i].work_hours,
        }
      }

      return reports;
    }

    self.getReports = function(search, limit) {
      if (search === undefined) {
        search = {};
      }

      var query = 'SELECT * FROM reports ';
      var params = [];
      var where = [];

      if (search.uuid != undefined) {
        where.push('uuid = ?');
        params.push(search.uuid);
      }

      if (search.uploaded != undefined) {
        if (search.uploaded) {
          where.push('upload_state = ? AND numPhotos = numPhotosUploaded');
          params.push(STATE_UPLOADED);
        }
        else {
          where.push('upload_state != ? OR numPhotos > numPhotosUploaded');
          params.push(STATE_UPLOADED);
        }
      }

      if (where.length > 0) {
        query += ' WHERE ' + where.join(' AND ') + ' ';
      }

      query += ' ORDER BY ts DESC';

      if (limit !== undefined) {
        query += ' LIMIT ?';
        params.push(limit);
      }

      return PhoneSQLite.fetchrows(DB_NAME, query, params)
        .then(getStatusesForReports)
        .then(getConditionsForReports)
        .then(getTrailsForReports)
        .then(getPhotosForReports)
        .then(matchUploadStateToPhotos)
        .then(convertTrailWork)
        //.catch(function(err) {
        //  console.log(err.message);
        //  return $q.reject(err);
        //})
        ;
    }

    self.addReport = function(report) {
      var currentMember = TFCurrentMember.getCurrentMember();

      if (!report.trail) {
        return $q.when($q.reject(new Error('Trail is required')));
      }

      if (!currentMember) {
        return $q.when($q.reject(new Error('Not logged in')));
      }

      var uuid = uuidgen.v1();

      // save trail info first
        // then save the report
      return TFTrailsLocal.saveTrail(report.trail)
        // upload the report data...
        .then(function() {
          var reportData = {
              action:          'add',
              ts:              +new Date,
              trailid:         report.trail.trailid,
              status:          report.status.value,
              condition:       report.condition.value,
              description:     report.description,
              marker:          '',
              trail_work:      (report.work.worked ? 1 : 0),
              work_hours:      (report.work.worked ? report.work.numHours : 0),
              work_attendance: (report.work.worked ? report.work.numPeople : 0),
          };

          return self.addReportData(uuid, reportData);
        })
        // upload the photos...
        .then(function() {
          return self.addReportPhotos(uuid, report.photos);
        })
        .then(function() {
          // after you add a report, try and upload it in background
          setTimeout(self.uploadReports,0)
        })
        // and finally update uploaded count
        .finally(function() {
          self.refreshQueuedCount();
        });
    };

    self.addReportData = function(reportID, data) {
      var query = 'INSERT INTO reports (id, uuid, trailid, ts, status, condition, description, marker, userid, trail_work, work_hours, work_attendance) VALUES (?,?,?,?,?,?,?,?,?,?,?,?);';
      var params = [reportID, reportID, data.trailid, data.ts, data.status, data.condition, data.description, data.marker, data.userid, data.trail_work, data.work_hours, data.work_attendance];
      return PhoneSQLite.query(DB_NAME, query, params);
    }

    self.addReportPhotos = function(reportID, photos) {
      if (photos.length) {
        var newName = reportID+'__'+photos.length+'.jpg';
        var photo = photos[0];
        return PhoneFileSystem.copy(photo.path, cordova.file.dataDirectory, newName)
          .then(function(newPath) {
            // photo copied, insert into db
            return self.addReportPhoto(reportID, newName);
          })
          .then(function() {
            // photo copied, and inserted, move on to the rest of these
            return self.addReportPhotos(reportID, photos.slice(1));
          });
      }
    }

    self.addReportPhoto = function(reportID, name) {
      var uuid = uuidgen.v1();

      var query = 'INSERT INTO report_photos (uuid, reportid, reportuuid, name) VALUES (?,?,?,?)';
      var params = [uuid, reportID, reportID, name];
      return PhoneSQLite.query(DB_NAME, query, params)
        .then(function() {
          var query = 'UPDATE reports SET numPhotos = numPhotos+1 WHERE id =  ?';
          var params = [reportID];
          return PhoneSQLite.query(DB_NAME, query, params)
        })
        //.catch(function(err) {
        //  console.log(err.message);
        //  return $q.reject(err);
        //})
        ;
    }

    self.refreshQueuedCount = function() {
      var query = 'SELECT COUNT(*) AS count FROM reports ';
      query += ' WHERE CAST(upload_state AS integer) != CAST(? AS integer) OR numPhotos > numPhotosUploaded';
      var params = [STATE_UPLOADED];

      return PhoneSQLite.fetchrow(DB_NAME, query, params)
        .then(function(row) {
          if (row.count != self.numQueued) {
            $rootScope.$broadcast('ReportQueueUpdate', row.count);
            self.numQueued = row.count;
          }
        })
        ;
    }

    self.uploadReports = function() {
      var reports;
      var index = 0;

      return getReports();

      function getReports() {
        $rootScope.$broadcast('TFReportsLocal.uploadreports.start', reports);
        return self.getReports({uploaded: false})
          .then(function(rows) {
            reports = rows;
            return uploadNextReport();
          })
      }

      function uploadNextReport() {
        if (index < reports.length) {
          var report = reports[index];
          index++;

          return self.uploadReport(report)
            .finally(uploadNextReport)
            ;
        }
        else {
          return finish();
        }
      }

      function finish() {
        $rootScope.$broadcast('TFReportsLocal.uploadreports.end');
      }
    }

    self.uploadReport = function(report) {
      if (report.upload_state === STATE_UPLOADED) {
        console.log('skipping report because it has already been uploaded');
        return $q.when();
      }
      else if (report.upload_state === STATE_UPLOADING && (+new Date < report.upload_ts + UPLOADING_TIMEOUT) ) {
        console.log('skipping report because it is being uploaded');
        return $q.when();
      }
      else {
        $rootScope.$broadcast('TFReportsLocal.uploadreport.start', STATE_UPLOADING, report);

        return uploadReportData(report)
          .then(function() {
            return uploadReportPhotos(report.id, report.photos);
          })
          .then(function() {
            $rootScope.$broadcast('TFReportsLocal.uploadreport.end', STATE_UPLOADED, report);
          })
          .catch(function(err) {
            $rootScope.$broadcast('TFReportsLocal.uploadreport.error', STATE_ERROR, report, err);
            return $q.reject(err);
          })
          .finally(self.refreshQueuedCount)
          ;
      }
    }

    function uploadReportData(report) {
      if (report.data_upload_state === STATE_UPLOADED ) {
        console.log('skipping report data because it has already been uploaded');
        return $q.when();
      }
      else if (report.data_upload_state === STATE_UPLOADING && (+new Date < report.upload_ts + UPLOADING_TIMEOUT) ) {
        console.log('skipping report data because it is being uploaded');
        return $q.when();
      }
      else {
        var reportData = {
            action:          'add',
            trailid:         report.trail.trailid,
            status:          report.status.value,
            condition:       report.condition.value,
            description:     report.description,
            marker:          report.marker,
            userid:          report.userid,
            trail_work:      report.trail_work,
            work_hours:      report.work_hours,
            work_attendance: report.work_attendance,
            source:          'reportapp',
            uuid:            report.uuid,
        };

        $rootScope.$broadcast('TFReportsLocal.uploadreportdata.start', STATE_UPLOADING, report);

        return TFUploadLogging.startUpload('report', report.uuid)
          .then(function(uploadid) {
            return TFCurrentMember.postPrivateAPI('report', reportData)
              // the 406 error status isn't actually an error, it just means we've
              // accidentally sent this request twice, so this first catch checks for that
              // and sends the request along normally if it's 406 error, otherwise it
              // passes the error along to be hadnled like a normal error
              .catch(function(err) {
                TFUploadLogging.uploadFailed(uploadid, err.message, err.uuid);

                if ('status' in err && err.status == 406 && 'contentid' in err && err.contentid > 0) {
                  return { reportid: err.contentid };
                }
                else {
                  return $q.reject(err);
                }
              })

              // handle a good response
              .then(function(response) {
                report.id  = ''+response.reportid;

                var query  = 'UPDATE reports SET id = ?, upload_state = ?, upload_ts = ? WHERE uuid =  ?';
                var params = [report.id, STATE_UPLOADED, +new Date, report.uuid];
                return PhoneSQLite.query(DB_NAME, query, params).then(function() {
                  var query = 'UPDATE report_photos SET reportid = ? WHERE reportuuid =  ?';
                  var params = [report.id, report.uuid];
                  return PhoneSQLite.query(DB_NAME, query, params);
                });
              })

              .then(function() {
                $rootScope.$broadcast('TFReportsLocal.uploadreportdata.end', STATE_UPLOADED, report);
                TFUploadLogging.uploadSucceeded(uploadid);
              })

              // handle an actual error.  for right now, we just notify.
              .catch(function(err) {
                var state = err.message != TFAPI.COULD_NOT_REACH_SERVER_ERROR ? STATE_ERROR : STATE_QUEUED;

                var query  = 'UPDATE reports SET upload_state = ?, upload_ts = ? WHERE id =  ?';
                var params = [state, +new Date, report.uuid];
                return PhoneSQLite.query(DB_NAME, query, params)
                  .finally(function() {
                    $rootScope.$broadcast('TFReportsLocal.uploadreportdata.error', state, report, err);
                    return $q.reject(err);
                  })
              })
              ;
          });
      }
    }

    function uploadReportPhotos(reportID, photos) {
      if (photos.length) {
        var photo = photos[0];

        return uploadReportPhoto(reportID, photo)
          .then(function() {
            return uploadReportPhotos(reportID, photos.slice(1));
          });
      }
      else {
        return $q.when();
      }
    }

    function uploadReportPhoto(reportID, photo) {
      if (photo.upload_state === STATE_UPLOADED ) {
        console.log('skipping photo because it has already been uploaded');
        return $q.when();
      }
      else if (photo.upload_state === STATE_UPLOADING && (+new Date < photo.upload_ts + UPLOADING_TIMEOUT) ) {
        console.log('skipping photo because it is being uploaded');
        return $q.when();
      }
      else {
        $rootScope.$broadcast('TFReportsLocal.uploadreportphoto.start', STATE_UPLOADING, photo);

        var query = 'UPDATE report_photos SET upload_state = ?, upload_ts = ? WHERE reportuuid =  ? AND name = ?';
        var params = [STATE_UPLOADING, +new Date, photo.reportuuid, photo.name];
        return PhoneSQLite.query(DB_NAME, query, params)
          .then(function() {
            return TFUploadLogging.startUpload('report_photo', photo.name)
              .then(function(uploadid) {
                return TFCurrentMember.uploadPrivateAPI(photo.path, {reportid: reportID, uuid: photo.uuid})
                  // the 406 error status isn't actually an error, it just means we've
                  // accidentally sent this request twice, so this first catch checks for that
                  // and sends the request along normally if it's 406 error, otherwise it
                  // passes the error along to be hadnled like a normal error
                  .catch(function(err) {
                    TFUploadLogging.uploadFailed(uploadid, err.message, err.uuid);

                    if ('status' in err && err.status == 406 && 'contentid' in err && err.contentid > 0) {
                      return { photoid: err.contentid };
                    }
                    else {
                      return $q.reject(err);
                    }
                  })

                  .then(function() {
                    var query = 'UPDATE reports SET numPhotosUploaded = numPhotosUploaded+1 WHERE uuid =  ?';
                    var params = [photo.reportuuid];
                    return PhoneSQLite.query(DB_NAME, query, params);
                  })

                  .then(function() {
                    var query = 'UPDATE report_photos SET upload_state = ?, upload_ts = ? WHERE reportuuid =  ? AND name = ?';
                    var params = [STATE_UPLOADED, +new Date, photo.reportuuid, photo.name];
                    return PhoneSQLite.query(DB_NAME, query, params);
                  })

                  .then(function() {
                    $rootScope.$broadcast('TFReportsLocal.uploadreportphoto.end', STATE_UPLOADED, photo);
                    TFUploadLogging.uploadSucceeded(uploadid);
                  })

                  .catch(function(err) {
                    var state = err.message != TFAPI.COULD_NOT_REACH_SERVER_ERROR ? STATE_ERROR : STATE_QUEUED;

                    var query  = 'UPDATE report_photos SET upload_state = ?, upload_ts = ? WHERE reportuuid =  ? AND name = ?';
                    var params = [state, +new Date, photo.reportuuid, photo.name];
                    return PhoneSQLite.query(DB_NAME, query, params)
                      .finally(function() {
                        $rootScope.$broadcast('TFReportsLocal.uploadreportphoto.error', state, photo, err);
                        return $q.reject(err);
                      })
                  });
              })
          })
      }
    }

    // make sure that these functions will create necessary tables if
    // they don't exist
    self.addReportData  = PhoneSQLite.createTableIfDoesNotExistWrapper(self.addReportData,  DB_NAME, 'reports',       REPORTS_TABLE_SCHEMA);
    self.addReportPhoto = PhoneSQLite.createTableIfDoesNotExistWrapper(self.addReportPhoto, DB_NAME, 'report_photos', REPORT_PHOTOS_TABLE_SCHEMA);
    getPhotosForReports = PhoneSQLite.createTableIfDoesNotExistWrapper(getPhotosForReports, DB_NAME, 'report_photos', REPORT_PHOTOS_TABLE_SCHEMA);
  }])
