angular.module('starter.services')
  .service('TFTrailsLocal', ['$q', 'PhoneSQLite', function($q, PhoneSQLite) {
    var self = this;

    var DB_NAME = 'tf.db';
    var TRAIL_TABLE_SCHEMA = 'CREATE TABLE trails(trailid PRIMARY KEY, title, difficulty);';

    self.saveTrail = function(trail) {
      var query = 'INSERT INTO trails (trailid, title, difficulty) VALUES (?,?,?)';
      var params = [trail.trailid, trail.title, trail.difficulty];
      return PhoneSQLite.query(DB_NAME, query, params).catch(function(err) {
        if (err.message.indexOf('column trailid is not unique') === 0) {
          return;
        }
        else {
          return $q.reject(err);
        }
      });

    };

    self.getTrails = function(ids) {
      if (ids === undefined || ids === null || ids.constructor !== Array || ids.length < 1) {
        return $q.when([]);
      }

      var placeholders = [];
      for (var i = 0; i < ids.length; i++) {
        placeholders.push('?');
      }

      var query = 'SELECT * FROM trails WHERE trailid IN ('+placeholders.join(',')+')';

      return PhoneSQLite.query(DB_NAME, query, ids).then(function(results) {
        var trails = [];

        for (var i = 0; i < results.rows.length; i++) {
          trails.push(results.rows.item(i));
        }

        return trails;
      });
    }

    // make sure that these functions will create necessary tables if
    // they don't exist
    // TODO down the line, should we prime the database on first run?
    self.saveTrail = PhoneSQLite.createTableIfDoesNotExistWrapper(self.saveTrail, DB_NAME, 'trails', TRAIL_TABLE_SCHEMA);
  }]);
